//  Demo2 unit : Bezier curves and smothing algorithm
//     Unit de dmonstration : Courbes de Bzier et algorithme de lissage
//     Jean-Yves Queinec    j.y.q@wanadoo.fr
//     Keywords : Bezier curves Demo2

unit QbezierU;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, Buttons, Menus;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Btquitter: TButton;
    UpDown1: TUpDown;
    Edit1: TEdit;
    UpDown2: TUpDown;
    Edit2: TEdit;
    Label2: TLabel;
    Btnouveau: TButton;
    Image1: TImage;
    Showpoints: TCheckBox;
    RadioOpen: TRadioGroup;
    Button1: TButton;
    Button2: TButton;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    RadioSym: TRadioGroup;
    ComboBox1: TComboBox;
    StatusBar1: TStatusBar;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    Bezier1: TMenuItem;
    About1: TMenuItem;
    N2: TMenuItem;
    New1: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Button3: TButton;
    procedure BtquitterClick(Sender: TObject);
    procedure BtnouveauClick(Sender: TObject);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure FormCreate(Sender: TObject);
    procedure UpDown2Click(Sender: TObject; Button: TUDBtnType);
    procedure FormDestroy(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ShowpointsClick(Sender: TObject);
    procedure RadioOpenClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);    // smooth, lissage
    procedure Button2Click(Sender: TObject);
    procedure ComboBox1Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
  public
    // repaint Image1 : grid and curve. Lisser = true -> apply smooth algorithm
    // repeint Image1 : quadrillage et courbe. Si lisser aplique alog de lissage
    procedure PaintAll(lisser : boolean);
    // draws Bezier curve in NOTXOR Mode
    // dessine la courbe de Bzier en mode NOTXOR
    procedure DrawCurve;
    procedure Quadrillage; // background grid, quadrillage de fond
    procedure polygone(cotes : integer);
    procedure sinusoide;
    Procedure BBtoAA;
    function  Nearpoint(x1,y1, x2,y2: integer): boolean;
    procedure Symetries(n : integer);
    procedure Loadcurve(afilename : string);
  end;

var
  Form1: TForm1;

implementation
  uses UBezier;

{$R *.DFM}
var
  French : boolean;
  Bmpfond : Tbitmap;            { background bitmap with grid pattern  }
                                // Bitmap de fond avec quadrillage
  Opencurve : boolean;          { kind of curve  : true open, false closed }
                                // type de Courbe : true ouverte, false ferme
  cx, cy : integer;             { image centre , centre de l'image }
  NB1 : longint;                { Number of AA points, nombre de points AA }
  AA: array[0..64] of Tpoint;   { edge points, points de la courbe }
  NB2 : longint;                { number of points used for Bezier curve }
                                // nombre de points pour la courbe de Bezier
  BB: array[0..64*3] of Tpoint; { points of the Bezier curve }
                                // points de la courbe de Bzier
  Anglepoly : single;           { angle for regular polygon }
                                // angle pour polygone rgulier }
  Angles : array[0..64*3] of single; { memorize angles for central symmetry}
                                // conserve les angles pour symtrie totale
  Startangle : single;         { starting angle for current point startp}
  infoang  : array[0..2] of single;  // to draw a polygon, pour tracer un polygone
  drawing : boolean;            { mouse in action, souris active }
  Startp : integer;       { current BB point selected, point BB slectionn }
  Curvelength : integer;
  Curverect   : Trect;

{---- The Form ---}
procedure TForm1.FormCreate(Sender: TObject);
begin
  French := true;
  drawing := false;
  quadrillage;
  Opencurve := false;  // a circle from 4 points. Un cercle  partir de 4 points
  polygone(4);
  PaintAll(true);  // true -> smooth the curve,  lissage
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Bmpfond.free;
end;

procedure TForm1.BtquitterClick(Sender: TObject);  // exit
begin
  close;
end;

procedure TForm1.FormResize(Sender: TObject);
var
  oldcx, oldcy, dcx, dcy : integer;
  i : integer;
begin
  oldcx := cx;
  oldcy := cy;
  quadrillage;   // computes also the new center positions cx and cy
  dcx := cx-oldcx;
  dcy := cy-oldcy;
  image1.picture.bitmap.assign(bmpfond);  // hiddden image1 drawing bitmap
  For i := 0 to NB2 do
  begin
    BB[i].x := BB[i].x+dcx;
    BB[i].y := BB[i].y+dcy;
  end;
  BBtoAA;  // update AA array
  paintall(false);
end;

procedure TForm1.PaintAll(lisser : boolean);
var
 i : integer;
begin
  Image1.canvas.draw(0,0,bmpfond);
  if lisser then
  begin
    // copy AA array points to BB array
    NB2 := NB1*3;
    for i := 0 to NB1 do
    begin
       BB[i*3] := AA[i]; // interval is 3 points
    end;
    lissage(BB, NB2+1, Updown1.position);
    // Complete symmetry memorize angular positions in order to keep good precision
    // during successive points displacements
    IF Not opencurve then for i := 0 to NB2 do
    begin
      Angles[i] := arctangente(BB[i].x-cx, BB[i].y-cy);
      if i < 3 then infoang[i] := Angles[i]; // memorize angles
    end;
  end;
  DrawCurve;
end;

procedure Tform1.DrawCurve;   // draws Bezier curve and points
var
  i : integer;
  {-----}
  procedure unecroix(ax, ay : integer; acolor : tcolor);
  begin
    with image1.canvas do
    begin
      pen.color := acolor;
      moveto(ax-1, ay); lineto (ax+2, ay);
      moveto(ax, ay-1); lineto(ax, ay+2);
    end;
  end;
  {-----}
begin
  with image1.canvas do
  begin
    pen.color := cllime;
    if drawing then
    begin
      case startp mod 3 of
      0 : begin
            if startp < NB2 then
            begin
              moveto(BB[startp].x, BB[startp].y);
              lineto(BB[startp+1].x, BB[startp+1].y);
            end;
            if startp > 0 then
            begin
              moveto(BB[startp-1].x, BB[startp-1].y);
              lineto(BB[startp].x, BB[startp].y);
            end;
          end;
      1 : begin
            moveto(BB[startp-1].x, BB[startp-1].y);
            lineto(BB[startp].x, BB[startp].y);
          end;
      2 : begin
            moveto(BB[startp].x, BB[startp].y);
            lineto(BB[startp+1].x, BB[startp+1].y);
          end;
      end; // case
    end;
    // Bzier curve *** Warning delphi releases change.
    // note : drawbezier is too slow with large curves
    image1.canvas.pen.color := clblack;
    polybezier(image1.canvas.handle, BB, NB2+1);   //** Delphi3
  //  image1.canvas.polybezier(slice(BB, NB2+1));    //** Delphi4 Delphi5
  //  2 next lines to test gradient coloured polybezier curve
  //  Drawbezier(BB,NB2+1,image1.canvas,0,clnone,clnone,curvelength, curverect);
  //  Drawbezier(BB,Nb2+1,image1.canvas,3,clyellow, clred,curvelength, curverect);

    // points
    For i := 0 to NB2 do
    begin
      case i mod 3 of
      0 : begin
           pen.color := clblack;
           ellipse(BB[i].x-2, BB[i].y-2, BB[i].x+2, BB[i].y+2);
          end;
      1 : If showpoints.Checked then unecroix(BB[i].x, BB[i].y, clblue);
      2 : If showpoints.Checked then unecroix(BB[i].x, BB[i].y, clred);
      end;
    end;
    // this last instruction force paintbox to repaint because an API function
    // using the canvas handle of a timage component doesn't do that
  end;
end;


{---- Utility Fonctions ----------}

procedure Tform1.quadrillage;   // bmpfond grid pattern
var                             // grille de fond dans Bmpfond
  i : integer;
begin
  Bmpfond := tbitmap.create;
  Bmpfond.width   := image1.width;
  Bmpfond.height  := image1.height;
  with Bmpfond.canvas do
  begin
    brush.color := clwhite;
    fillrect(rect(0,0,Image1.width, Image1.height));
    cx := Image1.width div 2;
    cy := Image1.height div 2;
    for i := 1 to Image1.width div 10 do
    begin
      if i mod 5 = 0 then pen.color := $00B0E0FF else pen.color := $00F0F4FF;
      moveto(cx+i*5, 0); lineto(cx+i*5, Image1.height);
      moveto(cx-i*5, 0); lineto(cx-i*5, Image1.height);
    end;
    for i := 1 to Image1.height div 10 do
    begin
      if i mod 5 = 0 then pen.color := $00B0E0FF else pen.color := $00F0F4FF;
      moveto(0,cy+i*5); lineto(Image1.width,cy+i*5);
      moveto(0,cy-i*5); lineto(Image1.width,cy-i*5);
    end;
    pen.color:= $0080B0D0;
    moveto(0,cy); lineto(Image1.width,cy);
    moveto(cx,0); lineto(cx, Image1.height);
  end;
end;

// Regular Polygon from number of points is NB1
procedure Tform1.Polygone(cotes : integer);
const
  b = 160.0;    // radius, Rayon
var
  i : integer;
  a1 : single; // angle and radius
  cx, cy : integer;  // centre
begin
  cx := image1.width div 2;
  cy := image1.height div 2;
  NB1 := cotes;   // polygon is closed
  IF NB1 < 2 then exit;
  anglepoly := -2*pi / NB1;  // angle increment clockwise sens aiguilles montre
  a1 := pi / 2;              // start angle
  For i := 0 to NB1-1 do
  begin
    AA[i].x := cx + round(b*cos(a1));
    AA[i].y := cy - round(b*sin(a1)); // invert y  trigonometric coordinates
    a1 := a1+anglepoly;
  end;
  AA[NB1] := AA[0];    // close polygon
  // Choose the smooth coefficent to obtain a circle. Lissage pour cercle
  if NB1 = 3 then updown1.position := 120
  else if NB1 = 4 then updown1.position := 105
  else if NB1 = 5 then updown1.position := 90
  else updown1.position := 75
end;

procedure Tform1.Sinusoide;  // aligned points and sine curve
var
  i : integer;
  a0, a, b : integer;
  r : integer;
begin
  NB1 := Updown2.position;
  a := (Image1.width - 24) div (NB1+1);
  a0 := cx - a*(NB1 div 2);
  b := Image1.height*3 div 8;
  for i := 0 to NB1 do
  begin
    AA[i].x := a0+a*i;
    r := i mod 4;
    case r of
    0 : AA[i].y := cy;
    1 : AA[i].y := cy-b;
    2 : AA[i].y := cy;
    3 : AA[i].y := cy+b;
    end;
  end;
end;

procedure Tform1.BBtoAA;  // copy BB modified points to AA
var
  i, a : integer;
begin
  a := 0;
  for i := 0 to NB2 do
    if i mod 3 = 0 then
    begin
      AA[a].x := BB[i].x;
      AA[a].y := BB[i].y;
      inc(a);
    end;
end;

//----------------- Image1 mouse clics -------------------
function Tform1.Nearpoint(x1,y1, x2,y2: integer): boolean;
begin
  if (abs(x1-x2) < 4) and (abs(y1-y2) < 4) then result := true
  else result := false;
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i : integer;
begin
  For i := 0 to NB2 do
  begin
    if nearpoint(BB[i].x, BB[i].y, x, y)  then
    begin
      if french then statusbar1.panels[1].text := ' Point n '+inttostr(i)
                else statusbar1.panels[1].text := ' Point # '+inttostr(i);
      startp := i;
      IF NOT Opencurve and (startp = NB2) then startp := 0;
      IF Opencurve  and (startp = NB2-1) and
         nearpoint(BB[NB2-1].x, BB[NB2-1].y, BB[NB2].x, BB[NB2].y)  then
         startp := NB2;
      startangle := arctangente(BB[startp].x -cx, BB[startp].y -cy);
      drawing := true;
      Image1.Canvas.draw(0,0, bmpfond);
      image1.canvas.pen.mode := pmnotxor;
      DrawCurve;    // Notxor drawing erase
      break;
    end;
  end;
end;

Procedure Tform1.symetries(n : integer);  // process symmetrical points together
var                                       // traite points symtriques ensemble
  sym : integer;
  i : integer;
  m : integer;
  rox, roy, ro : single;
  a0, a : single;
begin
  if (n < 0) or (n > NB2) then exit;
  Case radiosym.itemindex of
  1: begin
     // centered symmetry for all point ok the same kind
     // curve point, next and prior contol points
     // symtrie centrale pour tous les points de mme type (point de la courbe,
     // point de contrle  droite ou point de contrle  gauche)
       if NOT opencurve then
       begin
         a0 := arctangente(BB[n].x-cx, BB[n].y-cy) - startangle;
         rox := BB[n].x-cx;
         roy := BB[n].y-cy;
         ro := sqrt(sqr(rox) + sqr(roy));  // same radius for all points
         m := n mod 3;                     // edge point or conrol point
         For i := 0 to NB2 do
         begin
           if i mod 3 = m  then
           begin
              a := angles[i] + a0 ;  // delta angle
              BB[i].x := Cx + round(ro*cos(a));
              BB[i].y := Cy - round(ro*sin(a));
           end;
         end;
       end;
     end;
  2: begin  // centered symmetry for 2 points, symtrie centrale pour 2 points
       if n < NB2 div 2 then sym := NB2 div 2 +n
       else sym := n - NB2 div 2;
       if sym < 0 then  sym := 0;
       if sym > NB2 then sym := NB2;
       if sym = n then exit;
       BB[sym].x := cx + cx- BB[n].x;
       BB[sym].y := cy + cy- BB[n].y;
       IF not opencurve then BB[NB2] := BB[0];
     end;
  3: begin  // vertical symmetry (involves 2 points, ne concerne que 2 points)
       sym := NB2-n;
       if sym = n then exit;
       BB[sym].x := cx + cx- BB[n].x;
       BB[sym].y := BB[n].y;
       IF not opencurve then BB[NB2] := BB[0];
     end;
  4: begin     // Horizontal symmetry (not used in this program)
       sym := NB2-n;
       if sym = n then exit;
       BB[sym].x := BB[n].x;
       BB[sym].y := cy + cy- BB[n].y;
       IF not opencurve then BB[NB2] := BB[0];
     end;
  end; // case
end;

procedure TForm1.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  statusbar1.panels[0].text := ' x = '+inttostr(x-cx)+' , y = '+inttostr(y-cy);
  IF drawing then
  begin
    DrawCurve;             // notxor erase, effacement
    BB[startp].x := x;
    BB[startp].y := y;
    IF NOT Opencurve and (startp = 0) then   // start and end points together
    begin                                    // points dbut et fin ensemble
      BB[NB2].x := BB[0].x;
      BB[NB2].y := BB[0].y;
    end;
    symetries(startp);
    BBtoAA;       // update original points location, mise  jour points originaux
    DrawCurve;    // notxor
  end;
end;

procedure TForm1.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i : integer;
begin
  IF not drawing then exit;
  image1.canvas.pen.mode := pmcopy;
  drawing := false;
  if Not opencurve and (radiosym.itemindex = 1) then      // update angles
    For i := 0 to Nb2 do Angles[i] := arctangente(BB[i].x-cx, BB[i].y-cy);
  PaintAll(false);
end;

//---------  section bouttons and updowns
procedure TForm1.BtnouveauClick(Sender: TObject);
begin
  if Radioopen.itemindex = 0 then
  begin
    Opencurve := false;
    polygone(updown2.position);
  end
  else
  begin
    Opencurve := true;
    sinusoide;
  end;
  PaintAll(true);
end;

// smooth factor change
procedure TForm1.UpDown1Click(Sender: TObject; Button: TUDBtnType);
begin
  Image1.Canvas.draw(0,0, bmpfond);
  PaintAll(true);
end;

//  Number of segments
procedure TForm1.UpDown2Click(Sender: TObject; Button: TUDBtnType);
begin
  IF NOT Opencurve then polygone(updown2.position) else sinusoide;
  PaintAll(true);
end;

procedure TForm1.ShowpointsClick(Sender: TObject);
begin
  PaintAll(false);
end;

procedure TForm1.RadioOpenClick(Sender: TObject);
begin
  if radioOpen.itemindex = 0 then
  begin
    Opencurve := false;
    AA[NB1] := AA[0];
    BB[NB2] := AA[NB1];
  end
  else
  begin
    Opencurve := true;
    AA[NB1].x := AA[NB1].x+8;  // open the curve, ouvre la courbe
    AA[NB1].y := AA[NB1].y+8;
    BB[NB2] := AA[NB1];
  end;
  PaintAll(false);
end;

procedure TForm1.Button1Click(Sender: TObject);   // sprite
begin
  Drawbezier(BB,NB2+1,Image1.canvas,4,clyellow,clred,curvelength,curverect);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
   PaintAll(true);
end;

procedure TForm1.ComboBox1Click(Sender: TObject);

    procedure echange(n1, n2 : integer);
    var
      apt : tpoint;
    begin
      apt := BB[n2];
      BB[n2] := BB[n1];
      BB[n1] := apt;
    end;
begin
  case combobox1.itemindex of
  0 : // Heart, coeur
      if fileexists('Coeur.dat') then loadcurve('Coeur.dat');
  1 : // Bernouilli's lemniscate, lemniscate de Bernouilli
      if fileexists('Lemniscate.dat') then loadcurve('Lemniscate.dat');
  2 : // egg , oeuf
      if fileexists('oeuf.dat') then loadcurve('Oeuf.dat');
  3 : // Flower , Fleur
      if fileexists('Fleur.dat') then loadcurve('Fleur.dat');
  4 : // Jigsaw puzzle piece, Pice de puzzle
      if fileexists('Puzzle.dat') then loadcurve('Puzzle.dat');
  end;  // case
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
begin
  French := true;
  Form1.caption := 'Courbes de Bzier - Jean-Yves Quinec';
  Btquitter.caption  := 'Quitter';
  Btnouveau.caption  := 'Nouveau';
  Radioopen.caption  := 'Courbe';
  Radioopen.items[0] := 'Ferme';
  Radioopen.items[1] := 'Ouverte';
  Button1.caption    := 'Sprite';
  Button2.caption    := 'Lissage';
  Radiosym.caption   := 'Symtrie';
  Radiosym.items[0]  := 'Aucune';
  Radiosym.items[1]  := 'Complete';
  Radiosym.items[2]  := 'Centrale';
  Radiosym.items[3]  := 'Verticale';
  Showpoints.caption := 'Points de contrle';
  Combobox1.text     := 'Exemples';
  Combobox1.items[0] := 'Coeur';
  Combobox1.items[1] := 'Lemniscate de Bernouilli';
  Combobox1.items[2] := 'Oeuf';
  Combobox1.items[3] := 'Fleur';
  Combobox1.items[4] := 'Pice de puzzle'; 
  File1.caption      := '&Fichier';
  New1.caption       := '&Nouveau';
  Open1.caption      := '&Ouvrir';
  Save1.caption      := '&Enregistrer';
  Exit1.caption      := '&Quitter';
  Help1.caption      := '&Aide';
  Bezier1.caption    := '&Courbes de Bzier';
  About1.caption     := '&Tout sur QBezier';
end;

procedure TForm1.BitBtn2Click(Sender: TObject);
begin
  French := true;
  Form1.caption := 'Bezier Curves - Jean-Yves Quinec';
  Btquitter.caption  := 'Exit';
  Btnouveau.caption  := 'New';
  Radioopen.caption  := 'Curve';
  Radioopen.items[0] := 'Closed';
  Radioopen.items[1] := 'Opened';
  Button1.caption    := 'Sprite';
  Button2.caption    := 'Smooth';
  Radiosym.caption   := 'Symmetry';
  Radiosym.items[0]  := 'None';
  Radiosym.items[1]  := 'Complete';
  Radiosym.items[2]  := 'Centered';
  Radiosym.items[3]  := 'Vertical';
  Showpoints.caption := 'Control points';
  Combobox1.text     := 'Examples';
  Combobox1.items[0] := 'Heart';
  Combobox1.items[1] := 'Bernouilli''s lemniscate';
  Combobox1.items[2] := 'Egg';
  Combobox1.items[3] := 'Flower';
  Combobox1.items[4] := 'Puzzle piece';
  File1.caption      := '&File';
  New1.caption       := '&New';
  Open1.caption      := '&Open';
  Save1.caption      := '&Save';
  Exit1.caption      := '&Exit';
  Help1.caption      := '&Help';
  Bezier1.caption    := '&Bezier curves';
  About1.caption     := '&About QBezier';
end;

//----------- Menu and file management ------------------
procedure TForm1.New1Click(Sender: TObject);
begin
  Btnouveauclick(sender);
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Btquitterclick(sender);
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  if opendialog1.execute then loadcurve(opendialog1.filename);
end;

procedure Tform1.Loadcurve(afilename : string);
var
  fichier : textfile;
  s : string;
  i, ctr, posit : integer;
begin
  assignfile(fichier, afilename);
  reset(fichier);
  ctr := 0;
  while not eof(fichier) do
  begin
    inc(ctr);
    readln(fichier, s);
    if ctr = 1 then
    begin
      if copy(s,1,6) <> 'BEZIER' then
      begin
        if French then
          showmessage('Le fichier '+opendialog1.filename+
                      ' ne dbute pas par le mot cl BEZIER')
        else
          showmessage('File '+opendialog1.filename+
                      ' doesn''t begin with keyword BEZIER');
        break;
      end;
    end
    else
    if ctr = 2 then
    begin
      delete(s,1,7);
      NB2 := strtoint(s);
    end
    else
    begin
      posit := pos(' ', s);
      BB[ctr-3].x := cx+strtoint(copy(s,1,posit-1));
      delete(s,1,posit);
      BB[ctr-3].y := cy+strtoint(s);
    end;
  end;
  closefile(fichier);
  updown2.position := NB2 div 3;
  BBtoAA;
  opencurve := false;
  For i := 0 to Nb2 do Angles[i] := arctangente(BB[i].x-cx, BB[i].y-cy);
  Paintall(false);
end;

procedure TForm1.Save1Click(Sender: TObject);
var
  i : integer;
  s , s1, s2: string;
  fichier : Textfile;
begin
  if french then
    s := InputBox('Bcourbe','Donnez un Nom  la courbe', '')
  else
    s := InputBox('Bcourbe','Please Name the curve', '');
  if s = '' then exit;
  Savedialog1.filename := s+'.dat';
  if savedialog1.execute then
  begin
    assignfile(fichier, savedialog1.filename);
    rewrite(fichier);
    writeln(fichier,'BEZIER '+s);
    writeln(fichier,'Points '+ inttostr(NB2));
    for i := 0 to NB2 do
    begin
      s1 := inttostr(BB[i].x-cx);
      if s1 = '' then s1 := '0';  // avoid empty '' numbrer 0
      s2 := inttostr(BB[i].y-cy);
      if s2 = '' then s2 := '0';
      writeln(fichier, s1+' '+s2);
    end;
    closefile(fichier);
  end;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  // just compute curverect : Bmode greater than curve drawing Bmode values
  Drawbezier(BB,NB2+1,image1.canvas,0,clnone,clnone,curvelength, curverect);
  with image1.canvas do
  begin
    pen.color := clfuchsia;
    with curverect do
    begin
      moveto(left, top);
      lineto(right, top);
      lineto(right, bottom);
      lineto(left, bottom);
      lineto(left, top);
    end;
  end;
end;

end.
