// Bezier Demo Unit - Jean-Yves Quinec -  j.y.q@wanadoo.fr
// Spit a simple Bezier curve in two - Couper d'une courbe de Bzier en 2
// Keep the original curve appearance - Conserve l'apparence de la courbe origine
// Interpolate 2 Bezier curves  - Interpolation de 2 courbes de Bzier
// Gradient drawing - dessin avec dgrad

unit UBsplit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, Buttons;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    TrackBar1: TTrackBar;
    Label1: TLabel;
    Button3: TButton;
    Label3: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Label5: TLabel;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    Button4: TButton;
    PaintBox1: TPaintBox;
    CheckBox3: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure WMEraseBkgnd(var m : TWMEraseBkgnd);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure paintbox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure paintbox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure paintbox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CheckBox1Click(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);

  private
    procedure Background;  // draws background , dessine le quadrillage
    procedure DrawBB;      // draws first red curve, dessine la courbe rouge
    procedure DrawCC;      // fraws second blue curve, dessine la courbe bleue
    procedure DrawDD;
    // test click near (prs de)  a curve point or a control point
    Function  Nearpoint(ax, ay : integer): integer;
    procedure DrawControlpoints(Cancan: Tcanvas; ptcolor, linecolor : tcolor;
                                  n: integer; AA: array of Tpoint);
    procedure DrawCurvepoints(Cancan: Tcanvas; acolor : tcolor;
                                   n: integer; AA: array of Tpoint);
  public
  end;

var
  Form1: TForm1;

implementation

Uses UBezier;

{$R *.DFM}

Var
  french : boolean;
  bmp : Tbitmap;               // background bitmap
  cx, cy : integer;            // paintbox1 center
  NB : integer;                // number of curve points -1 = 1
  BB : array[0..3] of tpoint;  // Bezier simple curve
  CC : array[0..3] of tpoint;  // Second curve when split.
  DD : array[0..3] of tpoint;  // Interpolated curve
  cut : boolean;               // true when curve BB is split into BB and CC
  currentcurve : integer;      // current curve clicked -1= none 0 = BB 1 = CC
  currentpt : integer;         // current clicked point 0..3   BB or CC
  drawing  : boolean;          // mouse action
  cutpoint : integer;          // point cut with the mouse
  cutparam : single;   // memorize parametric t value when curve is split in two
  Tvalue   : single;   // current interpolate value
  curvelength : integer;
  curverect   : Trect;

Procedure TForm1.FormCreate(Sender: TObject);
begin
  french := false;
  background;
  cut     := false;
  drawing := false;
  Tvalue := 0.5;
  paintbox1.canvas.pen.mode := pmNotXor;  // all drawings in xormode
  button2Click(sender);                   // tout est dessin en mode NOtxor
end;

// avoid flicker
procedure TForm1.WMEraseBkgnd(var m : TWMEraseBkgnd);
begin
  m.Result := LRESULT(False);
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  Paintbox1.invalidate;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  bmp.free;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  close;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
begin
  background;     // repaint paintbox1
  drawBB;
  DrawCC;
  drawDD;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  bmp.free;
  bmp := tbitmap.create;
  bmp.width  := paintbox1.width;
  bmp.height := paintbox1.height;
  paintbox1.invalidate;
end;

procedure TForm1.Button2Click(Sender: TObject); // initialize
var
  i : integer;
begin
  background;   // computes center cx and cy
  NB := 4;
  BB[0].x := cx-200; BB[0].y := cy+100;
  BB[1].x := cx-150; BB[1].y := cy-150;
  BB[2].x := cx+150; BB[2].y := cy-150;
  BB[3].x := cx+200; BB[3].y := cy+100;
  for i := 0 to 3 do
  begin
    CC[i].x := 0;
    CC[i].y := 0;
  end;
  cut := false;     // only one curve not split
  drawBB;
  checkbox1.enabled := false;
  button3.enabled := false;
end;

procedure Tform1.DrawControlpoints(Cancan: Tcanvas; ptcolor, linecolor: tcolor;
                                    n: integer; AA: array of Tpoint);
var
  i : integer;
begin
  with cancan do
  begin
    pen.mode := pmnotxor;
    for i := 0 to n-1 do
    begin
      if linecolor <> clnone then
      begin
        pen.color := linecolor;
        if i mod 4 = 1 then
        begin
          Moveto(AA[i-1].x, AA[i-1].y);
          Lineto(AA[i].x, AA[i].y);
        end;
        if i mod 4 = 2 then
        begin
          Moveto(AA[i+1].x, AA[i+1].y);
          Lineto(AA[i].x, AA[i].y);
        end;
      end;
      if (i mod 4 = 1) or (i mod 4 = 2) then
      begin
        pen.color := ptcolor;
        Rectangle(AA[i].x-3, AA[i].y-3, AA[i].x+4, AA[i].y+4);
      end;
    end;
  end;
end;

procedure Tform1.DrawCurvepoints(Cancan: Tcanvas; acolor : tcolor;
                                   n: integer; AA: array of Tpoint);
var
  i : integer;
begin
  with cancan do
  begin
    pen.mode := pmnotxor;
    pen.color := acolor;
    for i := 0 to n-1 do
      if (i mod 4 = 0) or (i mod 4 = 3) then
       Ellipse(AA[i].x-3, AA[i].y-3, AA[i].x+3, AA[i].y+3);
  end;
end;

procedure Tform1.drawBB;     // First curve, red
begin
  DrawBezier(BB,4, paintbox1.canvas, 1, clred, clnone, curvelength, curverect);
  Drawcontrolpoints(paintbox1.canvas, clgreen, clsilver, 4, BB);
  Drawcurvepoints(paintbox1.canvas, clpurple, 4, BB);
end;

procedure Tform1.drawCC;    // second curve, blue
begin
  if not cut then exit;
  DrawBezier(CC, 4, paintbox1.canvas, 1, clblue, clnone, curvelength, curverect);
  Drawcontrolpoints(paintbox1.canvas, clgreen, clsilver, 4, CC);
  Drawcurvepoints(paintbox1.canvas, clNavy, 4, CC);
end;

procedure Tform1.drawDD;   // interpolated curve with gradient colors
begin                      // courbe interpole avec dgrad de couleurs
  if not cut then exit;
  if checkbox1.checked = false then exit;
  if Tvalue < 0.01 then Tvalue := 0.01
  else if Tvalue > 0.99 then Tvalue := 0.99;
  Binterpol(BB, CC, 4, Tvalue, DD);   // DD is the resuling array
  With paintbox1.canvas do
  begin
    if checkbox2.checked then pen.mode := pmcopy; // trace
    if checkbox3.checked then pen.width := 2;
    DrawBezier(DD, 4,paintbox1.canvas, 3, clteal, claqua, curvelength, curverect);
    pen.width := 1;
    pen.mode := pmNotxor;
    if NOT checkbox2.checked then Drawcurvepoints(paintbox1.canvas, clNavy, 4, DD);
  end;
end;

// Result = -1 when not near a curve or control point, otherwise near point number
// Rsltat = - 1 si pas prs d'un point de la courbe ou d'un point de contrle,
//                sinon n de point
Function Tform1.Nearpoint(ax, ay : integer): integer;
var
  i : integer;
  pt : tpoint;
  r : trect;
begin
  pt.x := ax;
  pt.y := ay;
  result := -1;
  currentcurve := -1;
  For i := 0 to 3 do
  begin
    r := rect(BB[i].x-8, BB[i].y-8, BB[i].x+8, BB[i].y+8);
    IF ptinrect (r, pt) then
    begin
      currentcurve := 0;
      result := i;     // 0..3
      exit;
    end;
  end;
  if cut then
  begin
    For i := 0 to 3 do
    begin
      r := rect(CC[i].x-8, CC[i].y-8, CC[i].x+8, CC[i].y+8);
      IF ptinrect (r, pt) then
      begin
        currentcurve := 1;
        result := i;     // 0..3
        exit;
      end;
    end;
  end;
end;

procedure TForm1.paintbox1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  p12, p01, p23, t1, t2 : Tpoint;
begin
  // click near BB or CC curve point or control points ?
  // clic  prs d'un point de BB ou CC ?
  currentpt := Nearpoint(x, y);
  if currentpt > -1 then
  begin
    drawing := true;
  end
  else
   // Test splitting - Test de la coupure en deux
  if Not cut and Nearbezier(BB, 4,x, y, 4, cutpoint, cutparam) then
  begin
    drawBB;
    cut := true;
    button3.enabled := true;
    checkbox1.enabled := true;
    // new bezier curve construction (see de Casteljau building)
    p01 := interpol(BB[cutpoint+1], BB[cutpoint], cutparam);
    p12 := interpol(BB[cutpoint+2], BB[cutpoint+1], cutparam);
    p23 := interpol(BB[cutpoint+3], BB[cutpoint+2], cutparam);
    t1  := interpol(p12, p01, cutparam);
    t2  := interpol(p23, p12, cutparam);
    // copy B[2] and B[3]
    CC[3] := BB[3];    // end   point  curve CC
    BB[3].x := x;      // new x end point curve BB
    BB[3].y := y;      // new y end point curve BB
    CC[0] := BB[3];    // start point  curve CC = end BB
    // new control points
    BB[1] := p01;
    CC[2] := p23;
    BB[2] := t1;
    CC[1] := t2;
    drawBB;
    drawCC;
    drawDD;
  end;
end;

procedure TForm1.paintbox1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  // cursor appearance management
  IF Not drawing  then
  begin
    if nearpoint(x, y) > -1 then paintbox1.cursor := crhandpoint
    else
    if not cut and Nearbezier(BB, 4, x, y, 4, cutpoint, cutparam)
    then
     paintbox1.cursor := crHsplit
    else
     paintbox1.cursor := crdefault;
  end
  else
  begin
    drawDD;
    case currentcurve of

      0 : begin
            drawBB;
            BB[currentpt].x := x;
            BB[currentpt].y := y;
            drawBB;
          end;
      1 : begin
           DrawCC;
           CC[currentpt].x := x;
           CC[currentpt].y := y;
           DrawCC;
         end;
    end;
    drawDD;
  end;
end;

procedure TForm1.paintbox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  drawing := false;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  DrawDD;   // create or erase
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  if checkbox1.checked then drawDD;
  Tvalue := trackbar1.position;
  Tvalue := Tvalue / Trackbar1.max;
  label1.caption := 't = '+formatfloat('0.00', tvalue);
  if checkbox1.checked then drawDD;
end;

procedure Tform1.background;  // build grid bitmap - Quadrillage
var
  i : integer;
begin
  bmp := Tbitmap.create;
  with paintbox1.canvas do
  begin
    brush.style := bssolid;
    brush.color := clwhite;
    fillrect(rect(0,0,paintbox1.width, paintbox1.height));
    cx := paintbox1.width div 2;
    cy := paintbox1.height div 2;
    for i := 1 to paintbox1.width div 10 do
    begin
      if i mod 5 = 0 then pen.color := $00B0E0FF else pen.color := $00F0F4FF;
      moveto(cx+i*5, 0); lineto(cx+i*5, paintbox1.height);
      moveto(cx-i*5, 0); lineto(cx-i*5, paintbox1.height);
    end;
    for i := 1 to paintbox1.height div 10 do
    begin
      if i mod 5 = 0 then pen.color := $00B0E0FF else pen.color := $00F0F4FF;
      moveto(0,cy+i*5); lineto(paintbox1.width,cy+i*5);
      moveto(0,cy-i*5); lineto(paintbox1.width,cy-i*5);
    end;
    pen.color:= $0080B0D0;
    moveto(0,cy); lineto(paintbox1.width,cy);
    moveto(cx,0); lineto(cx, paintbox1.height);
    font.size := 10;
    font.color := clblack;
    brush.style := bsclear;
  end;
end;

procedure TForm1.Button3Click(Sender: TObject); // Join
begin
  DrawBB;       // erase curves
  DrawCC;
  DrawDD;
  BB[3] := CC[3];
  DrawBB;       // only BB
  cut := false;
  checkbox1.checked := false;
  checkbox1.enabled := true;
  label1.visible    := true;
  trackbar1.position := 64;
  trackbar1.visible := true;
  button3.enabled := false;
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
begin
  if sender = Bitbtn1 then
  begin
    french := true;
    Form1.caption := 'Courbe de Bzier: coupure et interpolation - Jean-Yves Quinec';
    button1.caption := 'Quitter';
    button2.caption := 'Rinit.';
    button3.caption := 'Joindre';
    button4.caption := 'Rafraichir';
    checkbox1.caption := 'Interpoler';
    label3.caption := 'Utilisez aussi les flches';
    label5.caption := 'cliquez sur la courbe pour couper en 2';
  end
  else
  begin
    french := false;
     Form1.caption := 'Bezier curve : splitting and interpolation - Jean-Yves Quinec';
    button1.caption := 'Exit';
    button2.caption := 'Reset';
    button3.caption := 'Join';
    checkbox1.caption := 'Interpolate';
    label3.caption := 'Use also Arrows keys';
    label5.caption := 'Click on curve to split';
   end;
end;



end.
