// UCastel unit : simple Bezier curve. deCasteljau building
//                courbe de bzier simple - Construction de de Casteljau
// Jean-Yves Queinec    j.y.q@wanadoo.fr
unit UCastel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, Buttons;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    Image1: TImage;
    TrackBar1: TTrackBar;
    Label1: TLabel;
    Label3: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Label6: TLabel;
    Label7: TLabel;
    CheckBox1: TCheckBox;
    RadioGroup1: TRadioGroup;
    procedure WMEraseBkgnd(var m : TWMEraseBkgnd);
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TrackBar1Change(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
  private
    procedure Initcurve;
    procedure Background;  // draws background , dessine le quadrillage
    procedure DrawBB;      // draws first red curve, dessine la courbe rouge
    procedure UpdateCaptions;  // bezier parameter and and tagent angle
    // test click near (prs de)  a curve point or a control point
    Function  Nearpoint(ax, ay : integer): integer;
    procedure DrawControlpoints(AA: array of Tpoint; n: integer;
                                Cancan: Tcanvas; ptcolor, linecolor : tcolor);
    procedure DrawCurvepoints(AA: array of Tpoint; n : integer;
                               Cancan: Tcanvas; acolor : tcolor);
  public
  end;

var
  Form1: TForm1;

implementation

Uses UBezier;

{$R *.DFM}

Var
  french : boolean;
  cx, cy : integer;            // image1 center
  NB : integer;                // number of curve points -1 = 1
  BB : array[0..3] of tpoint;  // Bezier simple curve
  currentpt : integer;         // current clicked point 0..3
  drawing  : boolean;           // mouse action
  Tvalue : real;      // current  parametric t value
  dot : integer;        // dot dor draw
  curvelength : integer;
  curverect   : Trect;

Procedure TForm1.FormCreate(Sender: TObject);
begin
  french := false;
  drawing := false;
  dot := 1;  // line
  Tvalue := 0.5;
  Image1.canvas.pen.mode := pmNotXor;
  initcurve;
end;

// avoid Timage flicker
procedure TForm1.WMEraseBkgnd(var m : TWMEraseBkgnd);
begin
  m.Result := LRESULT(False);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  close;
end;

procedure TForm1.initcurve;
var
  i : integer;
begin
  // initialize
  background;   // computes cx and cy
  NB := 4;
  BB[0].x := cx-200; BB[0].y := cy+100;
  BB[1].x := cx-150; BB[1].y := cy-150;
  BB[2].x := cx+150; BB[2].y := cy-150;
  BB[3].x := cx+200; BB[3].y := cy+100;
end;

procedure Tform1.DrawControlpoints(AA: array of Tpoint; n: integer;
                 Cancan: Tcanvas; ptcolor, linecolor : tcolor);
var
  i : integer;
begin
  with cancan do
  begin
    pen.mode := pmnotxor;
    for i := 0 to n-1 do
    begin
      if linecolor <> clnone then
      begin
        pen.color := linecolor;
        if i mod 4 = 1 then
        begin
          Moveto(AA[i-1].x, AA[i-1].y);
          Lineto(AA[i].x, AA[i].y);
        end;
        if i mod 4 = 2 then
        begin
          Moveto(AA[i+1].x, AA[i+1].y);
          Lineto(AA[i].x, AA[i].y);
        end;
      end;
      if (i mod 4 = 1) or (i mod 4 = 2) then
      begin
        pen.color := ptcolor;
        Rectangle(AA[i].x-3, AA[i].y-3, AA[i].x+4, AA[i].y+4);
      end;
    end;
  end;
end;

procedure Tform1.updateCaptions;
var
  a : real;
begin
  label1.caption := 't = '+formatfloat('0.00', tvalue);
  a := TangentBezier(BB, 4 , 0, tvalue);
  a := (a*180) / pi;   // convert in degrees
  label4.caption := formatfloat('0.0', a)+'';
end;

procedure Tform1.DrawCurvepoints(AA: array of Tpoint; n : integer;
                                 Cancan: Tcanvas; acolor : tcolor);
var
  i : integer;
begin
  with cancan do
  begin
    pen.mode := pmnotxor;
    pen.color := acolor;
    for i := 0 to n-1 do
      if (i mod 4 = 0) or (i mod 4 = 3) then
       Ellipse(AA[i].x-3, AA[i].y-3, AA[i].x+3, AA[i].y+3);
  end;
end;

procedure Tform1.drawBB;
begin
  Drawbezier(BB, 4, image1.canvas, dot, clred, clblack, curvelength, curverect);
  label7.caption := inttostr(curvelength);
  Drawcontrolpoints(BB, 4, Image1.canvas, clgreen, clsilver);
  Drawcurvepoints(BB, 4, Image1.canvas, clpurple);
end;

// Result = -1 when not near a control point, otherwise point number
Function Tform1.Nearpoint(ax, ay : integer): integer;
var
  i : integer;
  pt : tpoint;
  r : trect;
begin
  pt.x := ax;
  pt.y := ay;
  result := -1;
  For i := 0 to 3 do
  begin
    r := rect(BB[i].x-8, BB[i].y-8, BB[i].x+8, BB[i].y+8);
    IF ptinrect (r, pt) then
    begin
      result := i;     // 0..3
      break;
    end;
  end;
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  p12, p01, p23, t1, t2 : Tpoint;
begin
  // Test clic on bezier curve point or control point
  currentpt := Nearpoint(x, y);
  if currentpt > -1 then drawing := true;
end;

procedure TForm1.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  // cursors management
  IF Not drawing  then
  begin
    if nearpoint(x, y) > -1 then image1.cursor := crhandpoint
    else
    Image1.cursor := crdefault;
  end
  else
  begin
    IF currentpt > - 1 then
    begin
      if checkbox1.checked then
       DeCasteljau(BB,4,image1.canvas,clsilver,clgray,clblue,0,Tvalue);
      drawBB;
      BB[currentpt].x := x;
      BB[currentpt].y := y;
      drawBB;
      if checkbox1.checked then
        DeCasteljau(BB,4,image1.canvas,clsilver,clgray,clblue,0,Tvalue);
    end;
  end;
end;

procedure TForm1.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  drawing := false;
  updatecaptions;
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  if checkbox1.checked then
    DeCasteljau(BB,4,image1.canvas,clsilver,clgray,clblue,0,Tvalue);
  Tvalue := trackbar1.position;
  Tvalue := Tvalue / 100;
  if checkbox1.checked then
    DeCasteljau(BB,4,image1.canvas,clsilver,clgray,clblue,0,Tvalue);
  UpdateCaptions;
end;

procedure Tform1.background;
var
  i : integer;
begin
  with image1.canvas do
  begin
    brush.style := bssolid;
    brush.color := clwhite;
    fillrect(rect(0,0,Image1.width, Image1.height));
    cx := Image1.width div 2;
    cy := Image1.height div 2;
    for i := 1 to Image1.width div 10 do
    begin
      if i mod 5 = 0 then pen.color := $00B0E0FF else pen.color := $00F0F4FF;
      moveto(cx+i*5, 0); lineto(cx+i*5, Image1.height);
      moveto(cx-i*5, 0); lineto(cx-i*5, Image1.height);
    end;
    for i := 1 to Image1.height div 10 do
    begin
      if i mod 5 = 0 then pen.color := $00B0E0FF else pen.color := $00F0F4FF;
      moveto(0,cy+i*5); lineto(Image1.width,cy+i*5);
      moveto(0,cy-i*5); lineto(Image1.width,cy-i*5);
    end;
    pen.color:= $0080B0D0;
    moveto(0,cy); lineto(Image1.width,cy);
    moveto(cx,0); lineto(cx, Image1.height);
    font.size := 10;
    font.color := clblack;
    brush.style := bsclear;
  end;
end;

procedure TForm1.FormResize(Sender: TObject);
var
  bmp : tbitmap;
begin
  bmp := tbitmap.create;
  try
    bmp.width  := image1.width;
    bmp.height := image1.height;
    image1.picture.bitmap.assign(bmp);
  finally
    bmp.free;
  end;
  background;
  drawBB;
  if checkbox1.checked then
    DeCasteljau(BB,4,image1.canvas,clsilver,clgray,clblue,0,Tvalue);
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
begin
  if sender = Bitbtn1 then
  begin
    french := true;
    Form1.caption := 'Courbe de Bzier simple - Jean-Yves Quinec';
    button1.caption := 'Quitter';
    label3.caption := 'Utilisez aussi les flches';
    label6.caption := 'longueur';
    Checkbox1.caption := 'Construction de de Casteljau';
    Radiogroup1.caption := 'Courbe';
    Radiogroup1.items[0] := 'Ligne';
    Radiogroup1.items[1] := 'Pointills';
  end
  else
  begin
    french := false;
    Form1.caption := 'Simple Bezier curve - Jean-Yves Quinec';
    button1.caption := 'Exit';
    label3.caption := 'Use also Arrows keys';
    label6.caption := 'length';
    Checkbox1.caption := 'de Casteljau''s building';
    Radiogroup1.caption := 'Curve';
    Radiogroup1.items[0] := 'Line';
    Radiogroup1.items[1] := 'Dots';
  end;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  // erase and create
    DeCasteljau(BB,4,image1.canvas,clsilver,clgray,clblue,0,Tvalue);
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  drawBB;     // erase
  dot := radiogroup1.itemindex+1;
  drawBB;     // draw
end;

end.
